import {
  path,
  fetch as kkFetch,
  clipboard,
  dialog,
  expose,
  HeadlessTemplateUiCommand,
  fs,
  kv,
  toast,
} from "@kksh/api/headless";
import {
  IMGUR_CLIENT_ID_KEY,
  URL_FORMAT_KEY,
  type URL_FORMATS,
} from "./constants";

function uint8ArrayToBase64(uint8Array: Uint8Array) {
  return btoa(
    Array.from(uint8Array)
      .map((byte) => String.fromCharCode(byte))
      .join("")
  );
}

const allowedImageExtensions = ["png", "jpg", "jpeg", "gif", "apng", "tiff"];

function formatUrl(url: string, format: (typeof URL_FORMATS)[number]) {
  switch (format) {
    case "plain":
      return url;
    case "markdown":
      return `![Image](${url})`;
    case "html":
      return `<img src="${url}" alt="Image" />`;
    default:
      return url;
  }
}

class ImgurUpload extends HeadlessTemplateUiCommand {
  uploadImage(
    clientId: string,
    base64Image: string
  ): Promise<string | undefined> {
    const formData = new FormData();
    formData.append("image", base64Image);
    return fetch("https://api.imgur.com/3/image", {
      method: "POST",
      body: formData,
      headers: {
        Authorization: `Client-ID ${clientId}`,
      },
    })
      .then((response) => response.json())
      .then((data) => {
        if (data.data.link) {
          return data.data.link as string;
        } else {
          toast.error("Failed to upload image to Imgur", {
            description: `Status: ${data.status}`,
          });
          return undefined;
        }
      })
      .catch((error) => {
        console.error(error);
        toast.error("Failed to upload image to Imgur", {
          description: `Error: ${error}`,
        });
        return undefined;
      });
  }

  uploadFromFile(clientId: string, path: string) {
    return fs
      .readFile(path)
      .then((uint8array) => this.uploadFromUint8Array(clientId, uint8array));
  }

  uploadFromUint8Array(clientId: string, uint8array: Uint8Array) {
    return this.uploadImage(clientId, uint8ArrayToBase64(uint8array));
  }

  async load() {
    const urlFormat = await kv.get(URL_FORMAT_KEY);
    return kv.get(IMGUR_CLIENT_ID_KEY).then(async (clientId) => {
      if (!clientId) {
        toast.error("Client ID is not set");
        return;
      }

      const hasFiles = await clipboard.hasFiles();
      if (hasFiles) {
        return clipboard.readFiles().then(async (files) => {
          if (files.length > 1) {
            const confirmed = await dialog.confirm(
              `Upload ${files.length} files to Imgur?`,
              {
                title: "Upload to Imgur",
                okLabel: "Upload",
                cancelLabel: "Cancel",
              }
            );
            if (!confirmed) {
              return toast.warning("Upload cancelled");
            }
          }
          const links: string[] = [];
          for (const file of files) {
            const ext = await path.extname(file);
            if (!allowedImageExtensions.includes(ext)) {
              toast.warning(`Unsupported file extension: ${ext}`, {
                description: file,
              });
              continue;
            }
            const link = await this.uploadFromFile(clientId, file);
            if (link) {
              links.push(link);
            }
          }
          return clipboard
            .writeText(
              links
                .map((link) => formatUrl(link, urlFormat ?? "plain"))
                .join("\n")
            )
            .then(() => {
              toast.success("Uploaded to Imgur (links copied to clipboard)", {
                description: `Uploaded ${links.length} images`,
              });
            });
        });
      }

      const hasImage = await clipboard.hasImage();
      if (hasImage) {
        return clipboard.readImageBase64().then((imageBase64) => {
          this.uploadImage(clientId, imageBase64).then((link) => {
            if (link) {
              return clipboard
                .writeText(formatUrl(link, urlFormat ?? "plain"))
                .then(() => {
                  toast.success(
                    "Uploaded to Imgur (link copied to clipboard)",
                    {
                      description: link,
                    }
                  );
                });
            }
          });
        });
      }
      const hasText = await clipboard.hasText();
      if (hasText) {
        return clipboard.readText().then(async (text) => {
          console.log(text);
          // check if text is url or file path
          if (text.startsWith("http")) {
            // url
            kkFetch(text)
              .then((response) => {
                console.log(response);
                return response.bytes();
              })
              .then((uint8arr) => {
                console.log(uint8arr);
                return this.uploadFromUint8Array(clientId, uint8arr).then(
                  (link) => {
                    if (link) {
                      return clipboard
                        .writeText(formatUrl(link, urlFormat ?? "plain"))
                        .then(() => {
                          toast.success(
                            "Uploaded to Imgur (link copied to clipboard)",
                            {
                              description: link,
                            }
                          );
                        });
                    }
                  }
                );
              })
              .catch((err) => {
                console.error(err);
                toast.error("Failed to upload image to Imgur", {
                  description: `Error: ${err}`,
                });
              });
          } else {
            // file path
            if (await fs.exists(text)) {
              this.uploadFromFile(clientId, text).then((link) => {
                if (link) {
                  return clipboard
                    .writeText(formatUrl(link, urlFormat ?? "plain"))
                    .then(() => {
                      toast.success(
                        "Uploaded to Imgur (link copied to clipboard)",
                        {
                          description: link,
                        }
                      );
                    });
                }
              });
            } else {
              toast.error("File does not exist", {
                description: text,
              });
            }
          }
        });
      }
    });
  }
}
expose(new ImgurUpload());
